@@ -7,11 +7,11 @@
 package com.jidesoft.plaf.basic;
 
 import com.jidesoft.icons.IconsFactory;
+import com.jidesoft.plaf.LookAndFeelFactory;
 import com.jidesoft.plaf.UIDefaultsLookup;
 import com.jidesoft.plaf.vsnet.VsnetMenuUI;
 import com.jidesoft.swing.*;
 import com.jidesoft.utils.SecurityUtils;
-import com.sun.java.swing.plaf.windows.WindowsLookAndFeel;
 
 import javax.swing.*;
 import javax.swing.event.MouseInputListener;
@@ -979,7 +979,7 @@ public class BasicJideSplitButtonUI extends VsnetMenuUI {
 
         int mnemonicIndex = menuItem.getDisplayedMnemonicIndex();
         // W2K Feature: Check to see if the Underscore should be rendered.
-        if (WindowsLookAndFeel.isMnemonicHidden()) {
+        if (LookAndFeelFactory.isMnemonicHidden()) {
             mnemonicIndex = -1;
         }
 
